
#include <SPI.h>
#include <SdFat.h>
#include <SamdAudio.h>
#include <Servo.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

SdFat SD;
SamdAudio AudioPlayer;

Servo servo1;  

Adafruit_SSD1306 display(-1); // Initialize I2C display. -1 indicates no reset pin needed.

int ammo = 5; // Amount of ammo remaining.
int weapon = 0; // Currently selected weapon.
int dnacheck = 0; // DNA check state.  If 0 then the weapon can fire normally. If anything else, we're waiting for the DNA check to end.
unsigned long dnastart = 0; // Time at which we began the DNA check. Used to animate the progress bar and text.

void setup() {

  pinMode(A3, INPUT_PULLDOWN); // Trigger
  pinMode(A4, INPUT_PULLDOWN); // DNA check
  pinMode(8, INPUT_PULLDOWN); // Weapon select
  
  // Pins 0..5 are the LEDs. I have used 1K resistors so I can drive 2 LEDs from each pin.
  for (int n=0; n<=5; n++) { pinMode(n, OUTPUT); }

  pinMode(26, OUTPUT); // Writing to pin 26 turns on both the NFET and the Tau's red status LED. It can also be accessed with PIN_LED_TXL.  

  servo1.attach(9); // Attach servo to pin 9.
  servo1.write(20); // Min & max angles will vary from servo to servo. For this servo valid angles are 20..160.  Outside this range the servo will continuously grind, use excessive power, and may eventually burn out.

  if (!SD.begin(10)) { // Attempt to open SD card with CS on pin 10.
    
    // Illuminate status LED momentarily (and trigger vibration motor and 3W LED) if we fail to read the SD card:
      digitalWrite(26, HIGH);
      delay(250);
      digitalWrite(26, LOW);
    
  }

  AudioPlayer.begin(22050);
  //AudioPlayer.play("fire.wav", 0); // Play startup sound.

  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);  // Initialize display with I2C address 0x3C. I don't know if all I2C OLED displays will use this address, but you can run an I2C scaning sketch to find your display address if need be. 
  display.clearDisplay(); // Clear the buffer. By default it has an Adafruit logo.
  display.display(); // Update the display. Not strictly necessary here.
       
}

void loop() {

  static int trigger = 0;
  static int select = 0;
  
  if (dnacheck == 0) { // If we are not doing the DNA check right now...
  
    if (digitalRead(A3) == HIGH) { // If trigger is being pulled...
  
      if (trigger == 0) { 
  
        trigger = 1;
              
        if (ammo > 0) { 
          AudioPlayer.play("fire.wav", 0);
          digitalWrite(26, HIGH); // Trigger flash and vibration.
          ammo = ammo - 1;
        } else {
          AudioPlayer.play("empty.wav", 0);
        }
        
      } 
      
    } else if (trigger == 1) {
      
      digitalWrite(26, LOW);
      trigger = 0;
        
    }
    
    if (digitalRead(8) == HIGH) { // Select weapon: 
   
      if (select == 0) {
        
        select = 1;
        //AudioPlayer.play("select.wav", 0);
        
        weapon = weapon + 1;
        if (weapon > 4) { weapon = 0; }
  
        switch (weapon) {
          case 0: // Standard
            AudioPlayer.play("1.wav", 0);
            servo1.write(20);
            break;
          case 1: // Heatseeker
            AudioPlayer.play("2.wav", 0);
            break;          
          case 2: // Armor piercing
            AudioPlayer.play("3.wav", 0);
            break;
          case 3: // Incendiary
            AudioPlayer.play("4.wav", 0);
            break;
          case 4: // High Ex
            AudioPlayer.play("5.wav", 0);
            servo1.write(160);
            break;
          /*
          case 5: // Ricochet
            AudioPlayer.play("6.wav", 0);
            servo1.write(20);
            break;
          */                                                       
        }
  
        ammo = 5;
  
      }
      
    } else if (select == 1) {
      select = 0;
    }

    if (digitalRead(A4) == HIGH) { // Did user press the button to initiate a DNA check?  
      dnacheck = 1;
      dnastart = millis();
      AudioPlayer.play("dna.wav", 0);
    }
    
  } else { // DNA check in progress!

    // It takes 1330ms for the bar to fill. During this time it says DNA CHECK.  After that, it says I.D. FAIL until 6735ms have passed.
    // Animation is handled in the drawDisplay() function.
    
  }
  
  drawDisplay();

  // Update ammo counter.
  
    // I have connected LEDs in pairs with a 1K resistor for each one between 3V and pins 0..4. Pin 5 has a single LED with a 470K resistor.
    // And because they're connected to 3V instead of ground, to turn then ON I need to pull the pin LOW to make a connection to ground instead of HIGH as would be the case if they were connected to ground.
    // Why did I connect them backwards? So I could use dual row pin headers on the board instead of having three rows of pins.
    // But if you are going to wire the LEDs directly to the board, which you would inside a blaster, you can connect them to the pin and ground instead!
      
  if (ammo >= 5) { digitalWrite(0, LOW); } else { digitalWrite(0, HIGH); }
  if (ammo >= 4) { digitalWrite(1, LOW); } else { digitalWrite(1, HIGH); }    
  if (ammo >= 3) { digitalWrite(2, LOW); } else { digitalWrite(2, HIGH); } 
  if (ammo >= 2) { digitalWrite(3, LOW); } else { digitalWrite(3, HIGH); } 
  if (ammo >= 1) { digitalWrite(4, LOW); } else { digitalWrite(4, HIGH); } 
  
  if (ammo == 0) { digitalWrite(5, LOW); } else { digitalWrite(5, HIGH); } // Red LED indicates EMPTY.
  
}

void drawDisplay() {

  unsigned long timepassed;
  
  display.setTextSize(1);
  display.setTextColor(WHITE);
  
  display.clearDisplay(); // Clear the buffer. By default it has an Adafruit logo.

  // Line 0,20 to 127,20 above ammo counter.
    display.drawLine(0, 20, 127, 20, WHITE);
  
  // Lines between ammo counters at x = 24, 48, 72, 96 from y=21 to 31
    display.drawLine(24, 21, 24, 30, WHITE);
    display.drawLine(48, 21, 48, 30, WHITE);
    display.drawLine(72, 21, 72, 30, WHITE);
    display.drawLine(96, 21, 96, 30, WHITE);
    
  // Diagonal lines (Doesn't appear to be ammo display.)
  // 4 lines drawn bottom to top, 6x6 w/ 2px seperation, starting 24 pixels in from right, 2 pixels down from top.
    display.drawLine(107,    4, 113,   0, WHITE);
    display.drawLine(107+4,  4, 113+4, 0, WHITE);
    display.drawLine(107+8,  4, 113+8, 0, WHITE);
    display.drawLine(107+12, 4, 113+12, 0, WHITE);

  // Ammo type / DNA check:
    display.setCursor(0,9);

    if (dnacheck > 0) {

      // It takes 1330ms for the bar to fill. During this time it says DNA CHECK.  After that, it says I.D. FAIL until 6735ms have passed.

      timepassed = millis()-dnastart;
      
      if (timepassed > 6735) {
        dnacheck = 0;    
      } else if (timepassed > 1330) {
        display.println("I.D. FAIL");
        display.fillRect(0, 0, 101, 4, WHITE);
      } else {
        display.println("DNA CHECK");
        display.fillRect(0, 0, 101*(timepassed/1330.0), 4, WHITE);
        // Length of bar is maxlength*(timepassed/1330.0)
        // Bar stops just before SEMI and SEMI begins at x=103.
      }
        
    } else if (ammo == 0) {
      display.println("EMPTY");  
    } else {
      
      switch (weapon) {
        case 0: // Standard
          display.println("STANDARD");
          break;
        case 1: // Heatseeker
          display.println("HEATSEEKER");
          break;          
        case 2: // Armor piercing
          display.println("ARMOR PIERCING");
          break;
        case 3: // Incendiary
          display.println("INCENDIARY");
          break;
        case 4: // High Ex
          display.println("HIGH EX");
          break;
        case 5: // Ricochet
          display.println("RICOCHET");
          break;                                           
      }

    }
    
    // Semi-auto or rapid-fire:
      display.setCursor(103,9);
      display.println("SEMI");
 
  // Distance
    display.setCursor(0,22);
    display.println("D");
    display.setCursor(4,22);
    display.println(":");
    display.setCursor(6+3,22);
    display.println("0");
    display.setCursor(6+3+4,22);
    display.println(".");
    display.setCursor(6+3+6+3,22);
    display.println("0");
        
  // Ammunition
    if (ammo == 0) {
      display.setCursor(25,22);
      display.println("00ap");       
      display.setCursor(49,22);
      display.println("00in");
      display.setCursor(73,22);
      display.println("00he");    
      display.setCursor(98,22);
      display.println("00fmj");      
    } else {
      display.setCursor(25,22);
      display.println("25ap");       
      display.setCursor(49,22);
      display.println("25in");
      display.setCursor(73,22);
      display.println("25he");    
      display.setCursor(98,22);
      display.println("50fmj");
    }
    
  //display.setTextColor(BLACK, WHITE); // 'inverted' text
  //display.println(3.141592);
    
  display.display(); // Update the display. 

}

